package org.kantega.jexmec.spring;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.ClassLoaderProvider;
import org.kantega.jexmec.PluginLoader;
import org.kantega.jexmec.PluginManagerListener;
import org.kantega.jexmec.ServiceLocator;
import org.kantega.jexmec.manager.DefaultPluginManager;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.AbstractFactoryBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;

import java.util.Collections;
import java.util.List;

/**
 * FactoryBean for easy creation and startup of a DefaultPluginManager in a Spring context.
 */
public class PluginManagerFactoryBean<P> extends AbstractFactoryBean implements ApplicationContextAware, ApplicationListener {
    private ApplicationContext applicationContext;


    private Class<P> pluginClass;

    private Class serviceClass;

    private ServiceLocator serviceLocator;

    private List<PluginLoader<P>> pluginLoaders = Collections.emptyList();

    private List<ClassLoaderProvider> classLoaderProviders = Collections.emptyList();

    private List<PluginManagerListener<P>> pluginManagerListeners = Collections.emptyList();

    private boolean addSystemClassLoaderPluginLoader = true;


    public Class getObjectType() {
        return DefaultPluginManager.class;
    }

    public boolean isSingleton() {
        return true;
    }

    public DefaultPluginManager createInstance() throws Exception {

        final DefaultPluginManager.Builder<P> builder =  DefaultPluginManager.buildFor(pluginClass);

        if (serviceLocator == null && serviceClass != null) {
            serviceLocator = new SpringServiceLocator(applicationContext, serviceClass);
        }

        if(serviceLocator != null) {
            builder.withServiceLocator(serviceLocator);
        }

        for (PluginLoader<P> loader : pluginLoaders) {
            builder.withPluginLoader(loader);
        }

        if(addSystemClassLoaderPluginLoader) {
            builder.withClassLoader(getClass().getClassLoader());
        }

        for(ClassLoaderProvider provider : classLoaderProviders) {
            builder.withClassLoaderProvider(provider);
        }


        DefaultPluginManager<P> manager = builder.build();

        for(PluginManagerListener<P> listener : pluginManagerListeners) {
            manager.addPluginManagerListener(listener);
        }

        return manager;
    }

    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }


    public void setPluginLoaders(List<PluginLoader<P>> pluginLoaders) {
        this.pluginLoaders = pluginLoaders;
    }

    public void setClassLoaderProviders(List<ClassLoaderProvider> classLoaderProviders) {
        this.classLoaderProviders = classLoaderProviders;
    }

    public void setServiceLocator(ServiceLocator serviceLocator) {
        this.serviceLocator = serviceLocator;
    }

    public void onApplicationEvent(ApplicationEvent event) {
        if (event instanceof ContextRefreshedEvent && event.getSource() == applicationContext) {
            try {
                DefaultPluginManager manager = (DefaultPluginManager) getObject();

                manager.start();
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    public void setServiceClass(Class serviceClass) {
        this.serviceClass = serviceClass;
    }

    public void setPluginClass(Class<P> pluginClass) {
        this.pluginClass = pluginClass;
    }


    public void setAddSystemClassLoaderProvider(boolean addSystemClassLoaderPluginLoader) {
        this.addSystemClassLoaderPluginLoader = addSystemClassLoaderPluginLoader;
    }

    public void setPluginManagerListeners(List<PluginManagerListener<P>> pluginManagerListeners) {
        this.pluginManagerListeners = pluginManagerListeners;
    }



    @Override
    public void afterPropertiesSet() throws Exception {
        if(pluginClass == null) {
            throw new IllegalArgumentException("You have not specified which pluginClass to use. Please set the pluginClass property to be your plugin class");
        }
        super.afterPropertiesSet();

    }

    @Override
    protected void destroyInstance(Object instance) throws Exception {
        DefaultPluginManager.class.cast(instance).stop();
    }
}
